let getAction = require('./connection_actions/action-dict');
let getParser = require('./payload_parsers/parser-dict');
let validator = require('./connection-validator');
let database = require('./database-wrapper');

const connectionRequestHandler = {
    add: function(requestBody){
        let user = requestBody.user;
        let provider = requestBody.provider;
        // contains authentication and specific info relating to the provider
        // (e.g. which repository or event to trigger from)
        let providerSettings = requestBody.providerSettings;
        let receiver = requestBody.receiver;
        // contains authentication and specific info relating to the receiver
        // (e.g. which server/channel/account to post to/as)
        let receiverSettings = requestBody.receiverSettings;
        let action = requestBody.action;
    
        let actionParser = getAction(action);
        let providerParser = getParser(provider);
        let receiverParser = getParser(receiver);
        let validation = validator(actionParser, providerParser, receiverParser, providerSettings, receiverSettings);
        if (validation === true) {
            // put variables above into database as a connection
            database.addConnection(user, action, provider, providerSettings, receiver, receiverSettings);
            let mesg = "Added " + action + " connection as user: " + user + " between " +
            provider + " and " + receiver + ". Provider options: " + providerSettings +
            ", Receiver options: " + receiverSettings;
            console.log(mesg);
            return { message: mesg };
        }
        else {
            console.log(validation);
            return { error: validation };
        }
    },
    
    edit: function(requestBody){
        let id = requestBody.id;
        let user = requestBody.user;
        let provider = requestBody.provider;
        let providerSettings = requestBody.providerSettings;
        let receiver = requestBody.receiver;
        let receiverSettings = requestBody.receiverSettings;
        let action = requestBody.action;
    
        // validate edit
        let owned = database.getConnection(id).user === user;
        let actionParser = getAction(action);
        let providerParser = getParser(provider);
        let receiverParser = getParser(receiver);
        let validation = validator(actionParser, providerParser, receiverParser, providerSettings, receiverSettings);
        if (owned !== true) {
            let mesg = "You do not have access to a connection with id: " + id + ".";
            return { error: mesg };
        }
        if (validation !== true) {
            console.log(validation);
            return { error: validation };
        }
        // since pre-existing connection with matching id is owned by the current user, set the
        // connection's options into the database with the given options
        database.editConnection(id, action, provider, providerSettings, receiver, receiverSettings);
        let mesg = "Edited connection " + id + " (" + action + ") as user: " + user +
        " between " + provider + " and " + receiver + ". Provider options: " + providerSettings
        + ", Receiver options: " + receiverSettings;
        console.log(mesg);
        return { message: mesg };
    },
    
    remove: function(requestBody){
        let id = requestBody.id;
        let user = requestBody.user;
    
        // validate remove
        let owned = database.getConnection(id).user === user;
        if (owned !== true) {
            let mesg = "You do not have access to a connection with id: " + id + ".";
            return { error: mesg };
        }
        // since pre-existing connection with matching id is owned by the current user, remove
        // the connection from the database
        database.removeConnection(id);
        let mesg = "Removed connection " + id + " as user: " + user;
        console.log(mesg);
        return { message: mesg };
    },
    
    update: function(requestBody){
        let user = requestBody.user;
        // send response containing all connections owned by the current user
        let userConnections = database.updateConnections(user);
        let mesg = "Updated user: " + user + "'s connections for client";
        return { message: mesg, userConnections };
    }
}

module.exports = connectionRequestHandler;